#!/usr/bin/env python
# -*- coding: UTF-8 -*-

##
## Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
##
## This software is published by the license of CPU-OS License, you can use and
## distibuted this software under this License. See CPU-OS Liense for more detail.
##
## You should have receive a copy of CPU-OS License. If not, please contact us
## by email <support_os@cpu-os.ac.cn>
##
##

import dbus
import dbus.service
import sys
import os
import json
import logging
import subprocess
from uefihandler import UEFIBootManager

# set dbus msg
global dbus_name
global dbus_path
global dbus_iface
dbus_name = "org.cdos.boot.manager"
dbus_path = "/org/cdos/boot/manager"
dbus_iface =  "org.cdos.boot.manager"

logging.basicConfig(format= '%(levelname)s %(asctime)-15s %(message)s file:%(filename)s line:%(lineno)d', filename='/var/log/bios-adaptor-daemon.log',level=logging.DEBUG)

class BiosAdaptorObject(dbus.service.Object):

    def __init__(self):
        self.session_bus = dbus.SystemBus()
        name = dbus.service.BusName(dbus_name, bus=self.session_bus)
        dbus.service.Object.__init__(self, name, dbus_path)
        self.uefibootmgr = UEFIBootManager()

    @dbus.service.method(dbus_iface, in_signature='', out_signature='s')
    def GetVersion(self):
        logging.debug("GetVersion")
        return "0.1"

    @dbus.service.method(dbus_iface, in_signature='', out_signature='b')
    def GetEFIBootStatus(self):
        return self.uefibootmgr.EFIFlag

    @dbus.service.method(dbus_iface, in_signature='', out_signature='a{sv}')
    def GetRepeatUEFIBootNum(self):
        if not self.uefibootmgr.EFIFlag:
            return {}
        return UEFIBootManager.getRepeatBootNum()

    @dbus.service.method(dbus_iface, in_signature='as', out_signature='as')
    def InActiveUEFIBootNumList(self, inactive_list=[]):
        if not self.uefibootmgr.EFIFlag:
            return inactive_list
        else:
            return UEFIBootManager.inactiveBootNumList(inactive_list)

    @dbus.service.method(dbus_iface, in_signature='as', out_signature='as')
    def ActiveUEFIBootNumList(self, active_list=[]):
        if not self.uefibootmgr.EFIFlag:
            return active_list
        else:
            return UEFIBootManager.activeBootNumList( active_list)

    @dbus.service.method(dbus_iface, in_signature='as', out_signature='as')
    def DeleteUEFIBootNumList(self, delete_list=[]):
        if not self.uefibootmgr.EFIFlag:
            return delete_list
        else:
            return UEFIBootManager.deleteBootNumList( delete_list)

    @dbus.service.method(dbus_iface, in_signature='', out_signature='b')
    def CanRebootToFirmwareSetup(self):
        if not self.uefibootmgr.EFIFlag:
            return False
        return self.uefibootmgr.can_reboot_to_firmware_setup_flag

    @dbus.service.method(dbus_iface, in_signature='', out_signature='b')
    def SetRebootToFirmwareSetup(self):
        if not self.uefibootmgr.EFIFlag:
            return False
        if not self.uefibootmgr.can_reboot_to_firmware_setup_flag:
            return False
        else:
            return self.uefibootmgr.setRebootToFirmwaresetup(True)

    @dbus.service.method(dbus_iface, in_signature='', out_signature='b')
    def RebootToFirmwareSetup(self):
        if not self.uefibootmgr.EFIFlag:
            return False
        if not self.uefibootmgr.can_reboot_to_firmware_setup_flag:
            return False
        else:
            if self.uefibootmgr.rebootToFirmwaresetup() == False:
                return False
            else:
                return True

    @dbus.service.method(dbus_iface, in_signature='a{sv}', out_signature='b')
    def ActiveFastBoot(self, active_status={}):
        return True

if __name__ == '__main__':
    # using glib
    import dbus.mainloop.glib
    dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
    from gi.repository import GObject
    GObject.threads_init()
    loop = GObject.MainLoop()
    biosadaptor = BiosAdaptorObject()
    loop.run()

