#!/usr/bin/env python
# -*- coding: UTF-8 -*-

##
## Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
##
## This software is published by the license of CPU-OS License, you can use and
## distibuted this software under this License. See CPU-OS Liense for more detail.
##
## You should have receive a copy of CPU-OS License. If not, please contact us
## by email <support_os@cpu-os.ac.cn>
##
##

import os,sys
import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk,GObject
import threading
import subprocess
import dbus

class UEFIBootManager:

    def __init__(self):
        firmware_setup_dbus_name = "org.freedesktop.login1"
        firmware_setup_dbus_path = "/org/freedesktop/login1"
        firmware_setup_dbus_iface = "org.freedesktop.login1.Manager"
        try:
            if os.path.isdir('/sys/firmware/efi/'):
                self.EFIFlag = True
            else:
                self.EFIFlag = False
        except Exception as e:
            self.EFIFlag = False

        if self.EFIFlag:
            self.firmware_setup_dbus_obj = dbus.SystemBus().get_object( firmware_setup_dbus_name, firmware_setup_dbus_path)
            self.firmware_setup_dbus_iface_obj = dbus.Interface( self.firmware_setup_dbus_obj, firmware_setup_dbus_iface)
            try:
                if self.firmware_setup_dbus_iface_obj.CanRebootToFirmwareSetup() == "yes":
                    self.can_reboot_to_firmware_setup_flag = True
                else:
                    self.can_reboot_to_firmware_setup_flag = False
            except Exception as e:
                self.can_reboot_to_firmware_setup_flag = False
        else:
            self.can_reboot_to_firmware_setup_flag = False

    @staticmethod
    def getUEFIBootList():
        res = {}
        boot_list = subprocess.check_output( ['efibootmgr', '-v']).split('\n')
        for boot_menu in boot_list:
            if ( not boot_menu.startswith('BootCurrent')) and ( not boot_menu.startswith('Timeout')) and ( not boot_menu.startswith('BootOrder')) and (boot_menu != ''):
                boot_num = boot_menu[0:8]
                if boot_menu[8] == '*':
                    active_statue = True
                else:
                    active_statue = False
                boot_label = boot_menu[10: -len(boot_menu.split('\t')[-1])].split('\t')[0]
                boot_msg = boot_menu.split('\t')[-1]
                res[boot_num] = {}
                res[boot_num]['label'] = boot_label
                res[boot_num]['status'] = active_statue
                res[boot_num]['msg'] = boot_msg
        return res

    @staticmethod
    def getUEFIBootListForLabel():
        uefi_boot_label_json = {}
        # uefi_boot_label_json = { boot_label: { boot_num: boot_msg} }
        uefi_boot_num_json = UEFIBootManager.getUEFIBootList()
        for boot_num in uefi_boot_num_json:
            boot_menu = uefi_boot_num_json[boot_num]
            boot_label = boot_menu['label']
            boot_msg = boot_menu['msg']
            boot_status = boot_menu['status']
            if boot_label not in uefi_boot_label_json:
                uefi_boot_label_json[boot_label] = {}
            uefi_boot_label_json[boot_label][boot_num] = {}
            uefi_boot_label_json[boot_label][boot_num]['msg'] = boot_msg
            uefi_boot_label_json[boot_label][boot_num]['status'] = boot_status
        return uefi_boot_label_json

    @staticmethod
    def getRepeatBootNum():
        # 获取详细信息完全相同的bootnum，状态为Inactive优先获取，重复的bootnum，保留一个不获取
        uefi_boot_label_json = UEFIBootManager.getUEFIBootListForLabel()
        repeat_boot_label_num_json = {}

        for boot_label in uefi_boot_label_json:
            if len(uefi_boot_label_json[boot_label]) > 1:
                repeat_boot_label_num_json[boot_label] = []
                boot_num_list = []
                repeat_boot_num_list = []

                for boot_num in uefi_boot_label_json[boot_label]:
                    if boot_num not in boot_num_list:
                        boot_num_list.append(boot_num)
                        boot_num_msg = uefi_boot_label_json[boot_label][boot_num]['msg']

                        for boot_num_2 in uefi_boot_label_json[boot_label]:
                            # 
                            if ( boot_num_2 not in boot_num_list) and ( uefi_boot_label_json[boot_label][boot_num_2]['msg'] == boot_num_msg):
                                # push inactive bootnum in repeat list,if one of them is active
                                if uefi_boot_label_json[boot_label][boot_num_2]['status'] == False:
                                    repeat_boot_num_list.append(boot_num_2)
                                else:

                                    # 获取bootname对应列表存在但repeat列表不存在的bootnum
                                    for boot_num_3 in list( set(boot_num_list) - set(repeat_boot_num_list)):
                                        # 如果内容一致
                                        if ( uefi_boot_label_json[boot_label][boot_num_3]['msg'] == uefi_boot_label_json[boot_label][boot_num_2]['msg']):
                                            # 如果boot_num_list中的bootnum(boot_num_3)为Active
                                            if uefi_boot_label_json[boot_label][boot_num_3]['status'] == True:
                                                repeat_boot_num_list.append(boot_num_2)
                                            # 如果boot_num_list中的bootnum(boot_num_3)为InActive
                                            else:
                                                repeat_boot_num_list.append(boot_num_3)
                                boot_num_list.append(boot_num_2)
                if repeat_boot_num_list != []:
                    repeat_boot_label_num_json[boot_label] = repeat_boot_num_list
        return repeat_boot_label_num_json

    @staticmethod
    def inactiveBootNumList( inactive_list = []):
        err_inactive_bootnum = []
        for inactive_bootnum in inactive_list:
            if os.system('efibootmgr -A -b ' + inactive_bootnum) != 0:
                err_inactive_bootnum.append(inactive_bootnum)
        return err_inactive_bootnum

    @staticmethod
    def activeBootNumList( active_list = []):
        err_active_bootnum = []
        for active_bootnum in active_list:
            if os.system('efibootmgr -a -b ' + active_bootnum) != 0:
                err_active_bootnum.append(active_bootnum)
        return err_active_bootnum

    @staticmethod
    def deleteBootNumList( delete_list = []):
        err_delete_bootnum = []
        for delete_bootnum in delete_list:
            if os.system('efibootmgr -B -b ' + delete_bootnum) != 0:
                err_delete_bootnum.append(delete_bootnum)
        return err_delete_bootnum

    def setRebootToFirmwaresetup(self, setFlag):
        try:
            if setFlag == True:
                self.firmware_setup_dbus_iface_obj.SetRebootToFirmwareSetup(True)
                return True
            else:
                return False
        except Exception as e:
            return False

    def rebootToFirmwaresetup(self):
        try:
            if os.system('systemctl reboot --firmware-setup') == 0:
                return True
            else:
                return False
        except Exception as e:
            return False
